-- --
-- 3D
-- --

-- Anlegen der Tabelle 

CREATE TABLE GeoDbLand3D (
  id     INTEGER,                          -- Identifikationsnummer
  name   VARCHAR(20),                      -- Objektname
  geo    MDSYS.SDO_GEOMETRY,               -- Objektgeometrie
  CONSTRAINT pk_gdbland3D PRIMARY KEY(id)  -- Primrschlssel
);

-- Einfgen der 3D-Daten

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (1,'Flaggenmast',MDSYS.SDO_GEOMETRY(3001,NULL,MDSYS.SDO_POINT_TYPE(10,11,5),NULL,NULL));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (2,'Steg',MDSYS.SDO_GEOMETRY(3002,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,2,1),MDSYS.SDO_ORDINATE_ARRAY(4,4,1, 5,7,1.5, 8,8,1)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (3,'Baumreihe',MDSYS.SDO_GEOMETRY(3005,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1,3),
                      MDSYS.SDO_ORDINATE_ARRAY(10,19,5, 12,19,10, 14,19,7)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (4,'Wall',MDSYS.SDO_GEOMETRY(3002,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,4,2, 1,2,2, 13,2,1),
                 MDSYS.SDO_ORDINATE_ARRAY(1,15,3,2.414,15.586,3,3,17,3,3.586,18.414,3,5,19,3,7,19,3)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (5,'Haus',MDSYS.SDO_GEOMETRY(3003,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,3),MDSYS.SDO_ORDINATE_ARRAY(9,9,7, 12,10,7)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (6,'Land',MDSYS.SDO_GEOMETRY(3007,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,1, 16,2003,1, 43,1003,1),
MDSYS.SDO_ORDINATE_ARRAY(1,1,1,19,1,1,19,20,1,1,20,1,1,1,1,4,5,1,3,10,1,7,14,1,13,15,1,17,12,1,16,8,1,13,5,1,9,3,1,4,5,1,7,7,1,12,7,1,13,11,1,10,13,1,7,10,1,7,7,1)));

COMMIT;

-- Metadaten bekannt geben

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEODBLAND3D','GEO', MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X',0,20,0.001),
                                                 MDSYS.SDO_DIM_ELEMENT('Y',0,20,0.001),
                                                 MDSYS.SDO_DIM_ELEMENT('Z',0,10,0.001)) ,
        NULL);
COMMIT;

-- Geometrieprfung

SELECT id FROM GeoDbLand3D WHERE SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,0.1) <> 'TRUE';

-- Geometrische Funktionen und Methoden

--- Abfrage Dimension:
SELECT name, g.geo.get_dims() FROM GeoDbLand3D g;

-- minimal umgebendes Rechteck:
SELECT SDO_GEOM.SDO_MBR(geo) FROM GeoDbLand3D WHERE name = 'Steg';

-- Lnge (auf 2D projiziert!):
SELECT name, SDO_GEOM.SDO_LENGTH(geo,0.1) FROM GeoDbLand3D WHERE name = 'Steg';


-- R-Baum als Index anlegen

CREATE INDEX GeoDbLand3D_geo_ix
ON GeoDbLand3D(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX
PARAMETERS ('SDO_INDX_DIMS=3');

-- Rechteckanfrage
SELECT name
FROM GeoDbLand3D
WHERE SDO_FILTER (geo, MDSYS.SDO_GEOMETRY(3003,NULL,NULL,
                    MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,3),
                    MDSYS.SDO_ORDINATE_ARRAY(7,7,1.5, 15,15,5))) = 'TRUE';


-- -------------------
-- SPATIAL DATA MINING
-- -------------------

-- Aufrumen:
DROP TABLE Clusters;
DELETE FROM USER_SDO_GEOM_METADATA WHERE table_name = 'CLUSTERS';
COMMIT;

-- Cluster-Tabelle anlegen:
CREATE TABLE Clusters (
  id   DECIMAL(3),                       -- ID des Clusters
  geo  SDO_GEOMETRY,                     -- Gebiet des Clusters
  CONSTRAINT pk_cluster PRIMARY KEY (id) -- Primrschlssel
);

-- Metadaten einfgen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('CLUSTERS','GEO', MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('Lnge',-180,180,1),
                                              MDSYS.SDO_DIM_ELEMENT('Breite',-90,90,1)) ,
        4326);
COMMIT;

-- Rumlichen Index anlegen:
CREATE INDEX Clusters_geo_ix
ON Clusters(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

-- Version ohne 'Outliers'

-- Cluster-Bestimmung:
INSERT INTO Clusters (id,geo)
  SELECT * FROM TABLE(SDO_SAM.SPATIAL_CLUSTERS('GEMEINDEN','CENTRUM',10,'TRUE'));

COMMIT;

SELECT id, geo FROM Clusters;

SELECT SUM(SDO_GEOM.SDO_AREA(c.geo,1,'unit=SQ_KM')) FROM Clusters c;

-- Geometrien auerhalb der Cluster bestimmen:
SELECT *
FROM (SELECT g.name, MIN( SDO_GEOM.SDO_DISTANCE (c.geo, g.centrum, 1, 'unit=M') ) AS dist
      FROM Gemeinden g CROSS JOIN Clusters c
      GROUP BY g.name)
WHERE Dist > 0;

-- Funktion zur fehlerfreien Zuordnung zu Clustern:
CREATE OR REPLACE FUNCTION BIN_GEOMETRY (
  data_geom IN SDO_GEOMETRY, tolerance IN NUMBER, 
  cluster_tablename IN VARCHAR2, cluster_colname IN VARCHAR2) RETURN NUMBER
IS
BEGIN
  RETURN SDO_SAM.BIN_GEOMETRY(data_geom,tolerance,cluster_tablename,cluster_colname);
EXCEPTION WHEN OTHERS THEN
  RETURN -1;
END;
/

-- Cluster-Zuordnung:
SELECT name, BIN_GEOMETRY(centrum,1,'CLUSTERS','GEO')
FROM Gemeinden
ORDER BY NAME;

-- Version mit 'Outliers'

DELETE FROM Clusters;

-- Cluster-Bestimmung:
INSERT INTO Clusters (id,geo)
  SELECT * FROM TABLE(SDO_SAM.SPATIAL_CLUSTERS('GEMEINDEN','CENTRUM',10,'FALSE'));

COMMIT;

SELECT id, geo FROM Clusters;

SELECT SUM(SDO_GEOM.SDO_AREA(c.geo,1,'unit=SQ_KM')) FROM Clusters c;

-- Geometrien auerhalb der Cluster bestimmen:
SELECT *
FROM (SELECT g.name, MIN( SDO_GEOM.SDO_DISTANCE (c.geo, g.centrum, 1, 'unit=M') ) AS dist
      FROM Gemeinden g CROSS JOIN Clusters c
      GROUP BY g.name)
WHERE Dist > 0;

-- Cluster-Zuordnung:
SELECT name, BIN_GEOMETRY(centrum,1,'CLUSTERS','GEO')
FROM Gemeinden
ORDER BY NAME;
